@@ -99,8 +99,6 @@ function configure_kuryr {
             iniset "$KURYR_CONFIG" cni_health_server cg_path \
                 "/system.slice/system-devstack.slice/devstack@kuryr-daemon.service"
         fi
-    else
-        iniset "$KURYR_CONFIG" cni_daemon daemon_enabled False
     fi
 
     create_kuryr_cache_dir
@@ -131,8 +129,6 @@ function configure_kuryr {
 }
 
 function generate_containerized_kuryr_resources {
-    local cni_daemon
-    cni_daemon=$1
     if [[ $KURYR_CONTROLLER_REPLICAS -eq 1 ]]; then
         KURYR_CONTROLLER_HA="False"
     else
@@ -262,7 +258,7 @@ function create_k8s_api_service {
     # Regardless of the octavia mode, the k8s API will be behind an L3 mode
     # amphora driver loadbalancer
     create_load_balancer_member "$(hostname)" "$address" "$api_port" \
-        default/kubernetes:${KURYR_K8S_API_LB_PORT} public-subnet "$lb_name" "$project_id"
+        default/kubernetes:${KURYR_K8S_API_LB_PORT} ${KURYR_NEUTRON_DEFAULT_EXT_SVC_SUBNET} "$lb_name" "$project_id"
 }
 
 function configure_neutron_defaults {
@@ -294,14 +290,12 @@ function configure_neutron_defaults {
     pod_subnet_id="$(openstack subnet show -c id -f value \
         "${KURYR_NEUTRON_DEFAULT_POD_SUBNET}")"
 
-    local use_octavia
-    use_octavia=$(trueorfalse True KURYR_K8S_LBAAS_USE_OCTAVIA)
     create_k8s_subnet "$project_id" \
                       "$KURYR_NEUTRON_DEFAULT_SERVICE_NET" \
                       "$KURYR_NEUTRON_DEFAULT_SERVICE_SUBNET" \
                       "$subnetpool_id" \
                       "$router" \
-                      "$use_octavia"
+                      "True"
     service_subnet_id="$(openstack subnet show -c id -f value \
         "${KURYR_NEUTRON_DEFAULT_SERVICE_SUBNET}")"
 
@@ -343,15 +337,13 @@ function configure_neutron_defaults {
         --remote-ip "$service_cidr" --ethertype IPv4 --protocol udp \
         "$service_pod_access_sg_id"
 
-    if [[ "$use_octavia" == "True" && \
-          "$KURYR_K8S_OCTAVIA_MEMBER_MODE" == "L3" ]]; then
+    if [[ "$KURYR_K8S_OCTAVIA_MEMBER_MODE" == "L3" ]]; then
         if [ -n "$sg_ids" ]; then
             sg_ids+=",${service_pod_access_sg_id}"
         else
             sg_ids="${service_pod_access_sg_id}"
         fi
-    elif [[ "$use_octavia" == "True" && \
-            "$KURYR_K8S_OCTAVIA_MEMBER_MODE" == "L2" ]]; then
+    elif [[ "$KURYR_K8S_OCTAVIA_MEMBER_MODE" == "L2" ]]; then
         # In case the member connectivity is L2, Octavia by default uses the
         # admin 'default' sg to create a port for the amphora load balancer
         # at the member ports subnet. Thus we need to allow L2 communication
@@ -432,30 +424,30 @@ function configure_neutron_defaults {
             --description "allow imcp traffic from everywhere to default namespace" \
             --ethertype IPv4 --protocol icmp "$allow_namespace_sg_id"
 
+        iniset "$KURYR_CONFIG" namespace_sg sg_allow_from_namespaces "$allow_namespace_sg_id"
+        iniset "$KURYR_CONFIG" namespace_sg sg_allow_from_default "$allow_default_sg_id"
+    fi
+
+    if [[ "$KURYR_SG_DRIVER" == "namespace" || "$KURYR_SG_DRIVER" == "policy" ]]; then
         # NOTE(ltomasbo): As more security groups and rules are created, there
         # is a need to increase the quota for it
          openstack --os-cloud devstack-admin --os-region "$REGION_NAME" \
              quota set --secgroups 100 --secgroup-rules 100 "$project_id"
-
-
-        iniset "$KURYR_CONFIG" namespace_sg sg_allow_from_namespaces "$allow_namespace_sg_id"
-        iniset "$KURYR_CONFIG" namespace_sg sg_allow_from_default "$allow_default_sg_id"
     fi
+
     if [ -n "$OVS_BRIDGE" ]; then
         iniset "$KURYR_CONFIG" neutron_defaults ovs_bridge "$OVS_BRIDGE"
     fi
     iniset "$KURYR_CONFIG" neutron_defaults external_svc_net "$ext_svc_net_id"
     iniset "$KURYR_CONFIG" octavia_defaults member_mode "$KURYR_K8S_OCTAVIA_MEMBER_MODE"
     iniset "$KURYR_CONFIG" octavia_defaults sg_mode "$KURYR_K8S_OCTAVIA_SG_MODE"
-    if [[ "$use_octavia" == "True" ]]; then
-        # Octavia takes a very long time to start the LB in the gate. We need
-        # to tweak the timeout for the LB creation. Let's be generous and give
-        # it up to 20 minutes.
-        # FIXME(dulek): This might be removed when bug 1753653 is fixed and
-        #               Kuryr restarts waiting for LB on timeouts.
-        iniset "$KURYR_CONFIG" neutron_defaults lbaas_activation_timeout 1200
-        iniset "$KURYR_CONFIG" kubernetes endpoints_driver_octavia_provider "$KURYR_EP_DRIVER_OCTAVIA_PROVIDER"
-    fi
+    # Octavia takes a very long time to start the LB in the gate. We need
+    # to tweak the timeout for the LB creation. Let's be generous and give
+    # it up to 20 minutes.
+    # FIXME(dulek): This might be removed when bug 1753653 is fixed and
+    #               Kuryr restarts waiting for LB on timeouts.
+    iniset "$KURYR_CONFIG" neutron_defaults lbaas_activation_timeout 1200
+    iniset "$KURYR_CONFIG" kubernetes endpoints_driver_octavia_provider "$KURYR_EP_DRIVER_OCTAVIA_PROVIDER"
 }
 
 function configure_k8s_pod_sg_rules {
@@ -764,6 +756,11 @@ function run_k8s_kubelet {
         command="$command --fail-swap-on=false"
     fi
 
+    if is_service_enabled coredns; then
+        local k8s_resolv_conf
+        command+=" --cluster-dns=${HOST_IP} --cluster-domain=cluster.local"
+    fi
+
     wait_for "Kubernetes API Server" "$KURYR_K8S_API_URL"
     if [[ "$USE_SYSTEMD" = "True" ]]; then
         # If systemd is being used, proceed as normal
@@ -777,13 +774,94 @@ function run_k8s_kubelet {
     fi
 }
 
+function run_coredns {
+    local output_dir=$1
+    mkdir -p "$output_dir"
+    rm -f ${output_dir}/coredns.yml
+    cat >> "${output_dir}/coredns.yml" << EOF
+apiVersion: v1
+kind: ConfigMap
+metadata:
+  name: coredns
+  namespace: kube-system
+data:
+  Corefile: |
+    .:53 {
+        bind ${HOST_IP}
+        errors
+        kubernetes cluster.local in-addr.arpa ip6.arpa {
+           pods insecure
+           upstream
+           fallthrough in-addr.arpa ip6.arpa
+        }
+        proxy . /etc/resolv.conf
+        cache 30
+        loop
+        reload
+        loadbalance
+EOF
+    if [[ "$ENABLE_DEBUG_LOG_LEVEL" == "True" ]]; then
+        cat >> "${output_dir}/coredns.yml" << EOF
+        debug
+        log
+EOF
+    fi
+    cat >> "${output_dir}/coredns.yml" << EOF
+    }
+---
+apiVersion: extensions/v1beta1
+kind: Deployment
+metadata:
+  name: coredns
+  namespace: kube-system
+  labels:
+    k8s-app: coredns
+    kubernetes.io/cluster-service: "true"
+    kubernetes.io/name: "CoreDNS"
+spec:
+  replicas: 1
+  selector:
+    matchLabels:
+      k8s-app: coredns
+  template:
+    metadata:
+      labels:
+        k8s-app: coredns
+      annotations:
+        scheduler.alpha.kubernetes.io/critical-pod: ''
+        scheduler.alpha.kubernetes.io/tolerations: '[{"key":"CriticalAddonsOnly", "operator":"Exists"}]'
+    spec:
+      hostNetwork: true
+      containers:
+      - name: coredns
+        image: coredns/coredns
+        imagePullPolicy: Always
+        args: [ "-conf", "/etc/coredns/Corefile" ]
+        volumeMounts:
+        - name: config-volume
+          mountPath: /etc/coredns
+      dnsPolicy: Default
+      volumes:
+        - name: config-volume
+          configMap:
+            name: coredns
+            items:
+            - key: Corefile
+              path: Corefile
+EOF
+
+    /usr/local/bin/kubectl apply -f ${output_dir}/coredns.yml
+}
+
+
 function run_kuryr_kubernetes {
     local python_bin=$(which python)
+
     if is_service_enabled openshift-master; then
-        wait_for "OpenShift API Server" "$KURYR_K8S_API_LB_URL" \
+        wait_for "OpenShift API Server" "${KURYR_K8S_API_ROOT}" \
             "${OPENSHIFT_DATA_DIR}/master/ca.crt" 1200
     else
-        wait_for_ok_health "Kubernetes API Server" "${KURYR_K8S_API_LB_URL}/healthz" \
+        wait_for_ok_health "Kubernetes API Server" "${KURYR_K8S_API_ROOT}/healthz" \
             "${KURYR_HYPERKUBE_DATA_DIR}/kuryr-ca.crt" \
             "${KURYR_HYPERKUBE_DATA_DIR}/kuryr.key" \
             "${KURYR_HYPERKUBE_DATA_DIR}/kuryr.crt" \
@@ -856,9 +934,16 @@ function create_ingress_l7_router {
 }
 
 function configure_overcloud_vm_k8s_svc_sg {
+    local dst_port
     local project_id
     local security_group
 
+    if is_service_enabled octavia; then
+        dst_port=${KURYR_K8S_API_LB_PORT}
+    else
+        dst_port=${KURYR_K8S_API_PORT}
+    fi
+
     project_id=$(get_or_create_project \
         "$KURYR_NEUTRON_DEFAULT_PROJECT" default)
     security_group=$(openstack security group list \
@@ -866,14 +951,12 @@ function configure_overcloud_vm_k8s_svc_sg {
         awk '{if ($2=="default") print $1}')
     openstack --os-cloud devstack-admin --os-region "$REGION_NAME" \
         security group rule create --project "$project_id" \
-        --dst-port "$KURYR_K8S_API_LB_PORT" "$security_group"
+        --dst-port "$dst_port" "$security_group"
     openstack port set "$KURYR_OVERCLOUD_VM_PORT" --security-group service_pod_access
 }
 
 function update_tempest_conf_file {
 
-    local use_octavia="$1"
-
     if [[ "$KURYR_USE_PORT_POOLS" == "True" ]]; then
         iniset $TEMPEST_CONFIG kuryr_kubernetes port_pool_enabled True
     fi
@@ -892,14 +975,9 @@ function update_tempest_conf_file {
     if [[ "$KURYR_ENABLED_HANDLERS" =~ .*policy.* ]]; then
         iniset $TEMPEST_CONFIG kuryr_kubernetes network_policy_enabled True
     fi
-    if ! is_service_enabled kuryr-daemon; then
-        iniset $TEMPEST_CONFIG kuryr_kubernetes kuryr_daemon_enabled False
-    fi
     # NOTE(yboaron): Services with protocol UDP are supported in Kuryr
-    # starting from Stein release and only for Octavia
-    if [[ "$use_octavia" == "True" ]]; then
-        iniset $TEMPEST_CONFIG kuryr_kubernetes test_udp_services True
-    fi
+    # starting from Stein release
+    iniset $TEMPEST_CONFIG kuryr_kubernetes test_udp_services True
     if [[ "$KURYR_CONTROLLER_HA" == "True" ]]; then
         iniset $TEMPEST_CONFIG kuryr_kubernetes ap_ha True
     fi
@@ -942,10 +1020,13 @@ if [[ "$1" == "stack" && "$2" == "extra" ]]; then
                                      subnet show "$KURYR_NEUTRON_DEFAULT_SERVICE_SUBNET" \
                                      -c cidr -f value)
             k8s_api_clusterip=$(_cidr_range "$service_cidr" | cut -f1)
-            # NOTE(dulek): KURYR_K8S_API_LB_URL will be a global to be used by next
-            #              deployment phases.
-            KURYR_K8S_API_LB_URL="https://${k8s_api_clusterip}:${KURYR_K8S_API_LB_PORT}"
-            iniset "$KURYR_CONFIG" kubernetes api_root ${KURYR_K8S_API_LB_URL}
+            # NOTE(mrostecki): KURYR_K8S_API_ROOT will be a global to be used by next
+            #                  deployment phases.
+            KURYR_K8S_API_ROOT=${KURYR_K8S_API_URL}
+            if is_service_enabled octavia; then
+                KURYR_K8S_API_ROOT="https://${k8s_api_clusterip}:${KURYR_K8S_API_LB_PORT}"
+            fi
+            iniset "$KURYR_CONFIG" kubernetes api_root ${KURYR_K8S_API_ROOT}
         else
             iniset "$KURYR_CONFIG" kubernetes api_root '""'
         fi
@@ -1037,11 +1118,7 @@ if [[ "$1" == "stack" && "$2" == "extra" ]]; then
     KURYR_FORCE_IMAGE_BUILD=$(trueorfalse False KURYR_FORCE_IMAGE_BUILD)
     if is_service_enabled kuryr-kubernetes || [[ ${KURYR_FORCE_IMAGE_BUILD} == "True" ]]; then
         if [ "$KURYR_K8S_CONTAINERIZED_DEPLOYMENT" == "True" ]; then
-            if is_service_enabled kuryr-daemon; then
-                build_kuryr_containers True
-            else
-                build_kuryr_containers False
-            fi
+            build_kuryr_containers
         fi
     fi
 
@@ -1049,11 +1126,7 @@ if [[ "$1" == "stack" && "$2" == "extra" ]]; then
         /usr/local/bin/kubectl apply -f ${KURYR_HOME}/kubernetes_crds/kuryrnet.yaml
         /usr/local/bin/kubectl apply -f ${KURYR_HOME}/kubernetes_crds/kuryrnetpolicy.yaml
         if [ "$KURYR_K8S_CONTAINERIZED_DEPLOYMENT" == "True" ]; then
-            if is_service_enabled kuryr-daemon; then
-                generate_containerized_kuryr_resources True
-            else
-                generate_containerized_kuryr_resources False
-            fi
+            generate_containerized_kuryr_resources
         fi
         if [ "$KURYR_MULTI_VIF_DRIVER" == "npwg_multiple_interfaces" ]; then
             /usr/local/bin/kubectl apply -f ${KURYR_HOME}/kubernetes_crds/network_attachment_definition_crd.yaml
@@ -1062,18 +1135,14 @@ if [[ "$1" == "stack" && "$2" == "extra" ]]; then
 
 elif [[ "$1" == "stack" && "$2" == "test-config" ]]; then
     if is_service_enabled kuryr-kubernetes; then
-        # NOTE(dulek): This is so late, because Devstack's Octavia is unable
-        #              to create loadbalancers until test-config phase.
-        use_octavia=$(trueorfalse True KURYR_K8S_LBAAS_USE_OCTAVIA)
-        if [[ "$use_octavia" == "False" ]]; then
-            create_k8s_router_fake_service
-        fi
-        create_k8s_api_service
-        #create Ingress L7 router if required
-        enable_ingress=$(trueorfalse False KURYR_ENABLE_INGRESS)
+        if is_service_enabled octavia; then
+            create_k8s_api_service
+            #create Ingress L7 router if required
+            enable_ingress=$(trueorfalse False KURYR_ENABLE_INGRESS)
 
-        if [ "$enable_ingress" == "True" ]; then
-            create_ingress_l7_router
+            if [ "$enable_ingress" == "True" ]; then
+                create_ingress_l7_router
+            fi
         fi
 
         # FIXME(dulek): This is a very late phase to start Kuryr services.
@@ -1089,13 +1158,20 @@ elif [[ "$1" == "stack" && "$2" == "test-config" ]]; then
             run_kuryr_daemon
         fi
 
+        if is_service_enabled coredns; then
+            #Open port 53 so pods can reach the DNS server
+            sudo iptables -I INPUT 1 -p udp -m udp --dport 53 -j ACCEPT
+
+            run_coredns "${DATA_DIR}/kuryr-kubernetes"
+        fi
+
         # Needs kuryr to be running
         if is_service_enabled openshift-dns; then
             configure_and_run_registry
         fi
     fi
     if is_service_enabled tempest; then
-        update_tempest_conf_file "$use_octavia"
+        update_tempest_conf_file
     fi
 fi
 
