@@ -120,6 +120,31 @@ class NetworkPolicyDriver(base.NetworkPolicyDriver):
             return existing_pod_selector
         return False
 
+    def _add_default_np_rules(self, sg_id):
+        """Add extra SG rule to allow traffic from svcs and host.
+
+        This method adds the base security group rules for the NP security
+        group:
+        - Ensure traffic is allowed from the services subnet
+        - Ensure traffic is allowed from the host
+        """
+        default_cidrs = []
+        default_cidrs.append(utils.get_subnet_cidr(
+            config.CONF.neutron_defaults.service_subnet))
+        worker_subnet_id = config.CONF.pod_vif_nested.worker_nodes_subnet
+        if worker_subnet_id:
+            default_cidrs.append(utils.get_subnet_cidr(worker_subnet_id))
+        for cidr in default_cidrs:
+            default_rule = {
+                u'security_group_rule': {
+                    u'ethertype': 'IPv4',
+                    u'security_group_id': sg_id,
+                    u'direction': 'ingress',
+                    u'description': 'Kuryr-Kubernetes NetPolicy SG rule',
+                    u'remote_ip_prefix': cidr
+                }}
+            driver_utils.create_security_group_rule(default_rule)
+
     def create_security_group_rules_from_network_policy(self, policy,
                                                         project_id):
         """Create initial security group and rules
@@ -142,6 +167,7 @@ class NetworkPolicyDriver(base.NetworkPolicyDriver):
             # Create initial security group
             sg = self.neutron.create_security_group(body=security_group_body)
             sg_id = sg['security_group']['id']
+            driver_utils.tag_neutron_resources('security-groups', [sg_id])
             i_rules, e_rules = self.parse_network_policy_rules(policy, sg_id)
             for i_rule in i_rules:
                 sgr_id = driver_utils.create_security_group_rule(i_rule)
@@ -151,19 +177,8 @@ class NetworkPolicyDriver(base.NetworkPolicyDriver):
                 sgr_id = driver_utils.create_security_group_rule(e_rule)
                 e_rule['security_group_rule']['id'] = sgr_id
 
-            # NOTE(ltomasbo): Add extra SG rule to allow traffic from services
-            # subnet
-            svc_cidr = utils.get_subnet_cidr(
-                config.CONF.neutron_defaults.service_subnet)
-            svc_rule = {
-                u'security_group_rule': {
-                    u'ethertype': 'IPv4',
-                    u'security_group_id': sg_id,
-                    u'direction': 'ingress',
-                    u'description': 'Kuryr-Kubernetes NetPolicy SG rule',
-                    u'remote_ip_prefix': svc_cidr
-                }}
-            driver_utils.create_security_group_rule(svc_rule)
+            # Add default rules to allow traffic from host and svc subnet
+            self._add_default_np_rules(sg_id)
         except (n_exc.NeutronClientException, exceptions.ResourceNotReady):
             LOG.exception("Error creating security group for network policy "
                           " %s", policy['metadata']['name'])
@@ -202,7 +217,7 @@ class NetworkPolicyDriver(base.NetworkPolicyDriver):
         else:
             matching_pods = driver_utils.get_pods(pod_selector, namespace)
         for pod in matching_pods.get('items'):
-            if pod['status']['podIP']:
+            if pod['status'].get('podIP'):
                 pod_ip = pod['status']['podIP']
                 ns = pod['metadata']['namespace']
                 ips.append({'cidr': pod_ip, 'namespace': ns})
@@ -356,22 +371,24 @@ class NetworkPolicyDriver(base.NetworkPolicyDriver):
 
         return ingress_sg_rule_body_list, egress_sg_rule_body_list
 
+    def delete_np_sg(self, sg_id):
+        try:
+            self.neutron.delete_security_group(sg_id)
+        except n_exc.NotFound:
+            LOG.debug("Security Group not found: %s", sg_id)
+        except n_exc.Conflict:
+            LOG.debug("Security Group already in use: %s", sg_id)
+            # raising ResourceNotReady to retry this action in case ports
+            # associated to affected pods are not updated on time, i.e.,
+            # they are still using the security group to be removed
+            raise exceptions.ResourceNotReady(sg_id)
+        except n_exc.NeutronClientException:
+            LOG.exception("Error deleting security group %s.", sg_id)
+            raise
+
     def release_network_policy(self, netpolicy_crd):
         if netpolicy_crd is not None:
-            try:
-                sg_id = netpolicy_crd['spec']['securityGroupId']
-                self.neutron.delete_security_group(sg_id)
-            except n_exc.NotFound:
-                LOG.debug("Security Group not found: %s", sg_id)
-            except n_exc.Conflict:
-                LOG.debug("Security Group already in use: %s", sg_id)
-                # raising ResourceNotReady to retry this action in case ports
-                # associated to affected pods are not updated on time, i.e.,
-                # they are still using the security group to be removed
-                raise exceptions.ResourceNotReady(sg_id)
-            except n_exc.NeutronClientException:
-                LOG.exception("Error deleting security group %s.", sg_id)
-                raise
+            self.delete_np_sg(netpolicy_crd['spec']['securityGroupId'])
             self._del_kuryrnetpolicy_crd(
                 netpolicy_crd['metadata']['name'],
                 netpolicy_crd['metadata']['namespace'])
@@ -456,6 +473,9 @@ class NetworkPolicyDriver(base.NetworkPolicyDriver):
             LOG.exception("Kubernetes Client Exception deleting kuryrnetpolicy"
                           " CRD.")
             raise
+        except n_exc.NotFound:
+            LOG.debug("KuryrNetPolicy CRD Object not found: %s",
+                      netpolicy_crd_name)
 
     def affected_pods(self, policy, selector=None):
         if selector:
