@@ -44,47 +44,115 @@ def _get_namespace_labels(namespace):
     return namespaces['metadata'].get('labels')
 
 
-def _create_sg_rules(crd, pod, pod_selector, rule_block, crd_rules,
-                     direction, matched, namespace=None):
+def _create_sg_rule(sg_id, direction, cidr, port=None, namespace=None):
+    if port:
+        sg_rule = driver_utils.create_security_group_rule_body(
+            sg_id, direction, port.get('port'),
+            protocol=port.get('protocol'), cidr=cidr, namespace=namespace)
+    else:
+        sg_rule = driver_utils.create_security_group_rule_body(
+            sg_id, direction, port_range_min=1,
+            port_range_max=65535, cidr=cidr, namespace=namespace)
+
+    sgr_id = driver_utils.create_security_group_rule(sg_rule)
+
+    sg_rule['security_group_rule']['id'] = sgr_id
+    return sg_rule
+
+
+def _create_sg_rules(crd, pod, pod_selector, rule_block,
+                     crd_rules, direction, matched, namespace=None):
     pod_labels = pod['metadata'].get('labels')
 
     # NOTE (maysams) No need to differentiate between podSelector
     # with empty value or with '{}', as they have same result in here.
     if (pod_selector and
             driver_utils.match_selector(pod_selector, pod_labels)):
-
         matched = True
         pod_ip = driver_utils.get_pod_ip(pod)
         sg_id = crd['spec']['securityGroupId']
         if 'ports' in rule_block:
             for port in rule_block['ports']:
-                sg_rule = driver_utils.create_security_group_rule_body(
-                    sg_id, direction, port.get('port'),
-                    protocol=port.get('protocol'), cidr=pod_ip,
+                sg_rule = _create_sg_rule(
+                    sg_id, direction, cidr=pod_ip, port=port,
                     namespace=namespace)
-                sgr_id = driver_utils.create_security_group_rule(sg_rule)
-                sg_rule['security_group_rule']['id'] = sgr_id
                 crd_rules.append(sg_rule)
         else:
-            sg_rule = driver_utils.create_security_group_rule_body(
-                sg_id, direction,
-                port_range_min=1,
-                port_range_max=65535,
-                cidr=pod_ip,
-                namespace=namespace)
-            sgr_id = driver_utils.create_security_group_rule(sg_rule)
-            sg_rule['security_group_rule']['id'] = sgr_id
+            sg_rule = _create_sg_rule(
+                sg_id, direction, cidr=pod_ip, namespace=namespace)
             crd_rules.append(sg_rule)
     return matched
 
 
-def _parse_rules(direction, crd, pod):
-    policy = crd['spec']['networkpolicy_spec']
-
+def _parse_selectors_on_pod(crd, pod, pod_selector, namespace_selector,
+                            rule_block, crd_rules, direction):
+    matched = False
     pod_namespace = pod['metadata']['namespace']
     pod_namespace_labels = _get_namespace_labels(pod_namespace)
     policy_namespace = crd['metadata']['namespace']
 
+    if namespace_selector == {}:
+        matched = _create_sg_rules(crd, pod, pod_selector, rule_block,
+                                   crd_rules, direction, matched)
+    elif namespace_selector:
+        if (pod_namespace_labels and
+            driver_utils.match_selector(namespace_selector,
+                                        pod_namespace_labels)):
+            matched = _create_sg_rules(crd, pod, pod_selector,
+                                       rule_block, crd_rules,
+                                       direction, matched, pod_namespace)
+    else:
+        if pod_namespace == policy_namespace:
+            matched = _create_sg_rules(crd, pod, pod_selector, rule_block,
+                                       crd_rules, direction, matched,
+                                       pod_namespace)
+    return matched, crd_rules
+
+
+def _parse_selectors_on_namespace(crd, direction, pod_selector,
+                                  ns_selector, rule_block, crd_rules,
+                                  namespace):
+    ns_name = namespace['metadata'].get('name')
+    ns_labels = namespace['metadata'].get('labels')
+    sg_id = crd['spec']['securityGroupId']
+    matched = False
+
+    if (ns_selector and ns_labels and
+            driver_utils.match_selector(ns_selector, ns_labels)):
+        if pod_selector:
+            pods = driver_utils.get_pods(pod_selector, ns_name)
+            for pod in pods.get('items'):
+                pod_ip = driver_utils.get_pod_ip(pod)
+                if 'ports' in rule_block:
+                    for port in rule_block['ports']:
+                        matched = True
+                        crd_rules.append(_create_sg_rule(
+                            sg_id, direction, pod_ip, port=port,
+                            namespace=ns_name))
+                else:
+                    matched = True
+                    crd_rules.append(_create_sg_rule(
+                        sg_id, direction, pod_ip,
+                        namespace=ns_name))
+        else:
+            ns_cidr = driver_utils.get_namespace_subnet_cidr(namespace)
+            if 'ports' in rule_block:
+                for port in rule_block['ports']:
+                    matched = True
+                    crd_rules.append(_create_sg_rule(
+                        sg_id, direction, ns_cidr,
+                        port=port, namespace=ns_name))
+            else:
+                matched = True
+                crd_rules.append(_create_sg_rule(
+                    sg_id, direction, ns_cidr,
+                    namespace=ns_name))
+    return matched, crd_rules
+
+
+def _parse_rules(direction, crd, pod=None, namespace=None):
+    policy = crd['spec']['networkpolicy_spec']
+
     rule_direction = 'from'
     crd_rules = crd['spec'].get('ingressSgRules')
     if direction == 'egress':
@@ -92,29 +160,20 @@ def _parse_rules(direction, crd, pod):
         crd_rules = crd['spec'].get('egressSgRules')
 
     matched = False
-    rule_list = policy.get(direction, None)
+    rule_list = policy.get(direction, [])
     for rule_block in rule_list:
         for rule in rule_block.get(rule_direction, []):
             namespace_selector = rule.get('namespaceSelector')
             pod_selector = rule.get('podSelector')
-            if namespace_selector == {}:
-                if _create_sg_rules(crd, pod, pod_selector, rule_block,
-                                    crd_rules, direction, matched):
-                    matched = True
-            elif namespace_selector:
-                if (pod_namespace_labels and
-                    driver_utils.match_selector(namespace_selector,
-                                                pod_namespace_labels)):
-                    if _create_sg_rules(crd, pod, pod_selector, rule_block,
-                                        crd_rules, direction, matched,
-                                        pod_namespace):
-                        matched = True
-            else:
-                if pod_namespace == policy_namespace:
-                    if _create_sg_rules(crd, pod, pod_selector, rule_block,
-                                        crd_rules, direction, matched,
-                                        pod_namespace):
-                        matched = True
+            if pod:
+                matched, crd_rules = _parse_selectors_on_pod(
+                    crd, pod, pod_selector, namespace_selector,
+                    rule_block, crd_rules, direction)
+            elif namespace:
+                matched, crd_rules = _parse_selectors_on_namespace(
+                    crd, direction, pod_selector, namespace_selector,
+                    rule_block, crd_rules, namespace)
+
     return matched, crd_rules
 
 
@@ -156,21 +215,24 @@ class NetworkPolicySecurityGroupsDriver(base.PodSecurityGroupsDriver):
 
     def create_sg_rules(self, pod):
         LOG.debug("Creating sg rule for pod: %s", pod['metadata']['name'])
+        crd_pod_selectors = []
         knp_crds = driver_utils.get_kuryrnetpolicy_crds()
         for crd in knp_crds.get('items'):
             crd_selector = crd['spec'].get('podSelector')
 
-            i_matched, i_rules = _parse_rules('ingress', crd, pod)
-            e_matched, e_rules = _parse_rules('egress', crd, pod)
+            i_matched, i_rules = _parse_rules('ingress', crd, pod=pod)
+            e_matched, e_rules = _parse_rules('egress', crd, pod=pod)
 
             if i_matched or e_matched:
                 driver_utils.patch_kuryr_crd(crd, i_rules,
                                              e_rules, crd_selector)
+                crd_pod_selectors.append(crd_selector)
+        return crd_pod_selectors
 
     def delete_sg_rules(self, pod):
         LOG.debug("Deleting sg rule for pod: %s", pod['metadata']['name'])
         pod_ip = driver_utils.get_pod_ip(pod)
-
+        crd_pod_selectors = []
         knp_crds = driver_utils.get_kuryrnetpolicy_crds()
         for crd in knp_crds.get('items'):
             crd_selector = crd['spec'].get('podSelector')
@@ -205,11 +267,80 @@ class NetworkPolicySecurityGroupsDriver(base.PodSecurityGroupsDriver):
             if matched:
                 driver_utils.patch_kuryr_crd(crd, i_rules, e_rules,
                                              crd_selector)
+                crd_pod_selectors.append(crd_selector)
+        return crd_pod_selectors
 
     def update_sg_rules(self, pod):
         LOG.debug("Updating sg rule for pod: %s", pod['metadata']['name'])
-        self.delete_sg_rules(pod)
-        self.create_sg_rules(pod)
+        crd_pod_selectors = []
+        crd_pod_selectors.extend(self.delete_sg_rules(pod))
+        crd_pod_selectors.extend(self.create_sg_rules(pod))
+        return crd_pod_selectors
+
+    def delete_namespace_sg_rules(self, namespace):
+        ns_name = namespace['metadata']['name']
+        LOG.debug("Deleting sg rule for namespace: %s",
+                  ns_name)
+
+        knp_crds = driver_utils.get_kuryrnetpolicy_crds()
+        for crd in knp_crds.get('items'):
+            crd_selector = crd['spec'].get('podSelector')
+            ingress_rule_list = crd['spec'].get('ingressSgRules')
+            egress_rule_list = crd['spec'].get('egressSgRules')
+            i_rules = []
+            e_rules = []
+
+            matched = False
+            for i_rule in ingress_rule_list:
+                LOG.debug("Parsing ingress rule: %r", i_rule)
+                rule_namespace = i_rule.get('namespace', None)
+
+                if rule_namespace and rule_namespace == ns_name:
+                    matched = True
+                    driver_utils.delete_security_group_rule(
+                        i_rule['security_group_rule']['id'])
+                else:
+                    i_rules.append(i_rule)
+
+            for e_rule in egress_rule_list:
+                LOG.debug("Parsing egress rule: %r", e_rule)
+                rule_namespace = e_rule.get('namespace', None)
+
+                if rule_namespace and rule_namespace == ns_name:
+                    matched = True
+                    driver_utils.delete_security_group_rule(
+                        e_rule['security_group_rule']['id'])
+                else:
+                    e_rules.append(e_rule)
+
+            if matched:
+                driver_utils.patch_kuryr_crd(
+                    crd, i_rules, e_rules, crd_selector)
+
+    def create_namespace_sg_rules(self, namespace):
+        kubernetes = clients.get_kubernetes_client()
+        ns_name = namespace['metadata']['name']
+        LOG.debug("Creating sg rule for namespace: %s", ns_name)
+        namespace = kubernetes.get(
+            '{}/namespaces/{}'.format(constants.K8S_API_BASE, ns_name))
+        knp_crds = driver_utils.get_kuryrnetpolicy_crds()
+        for crd in knp_crds.get('items'):
+            crd_selector = crd['spec'].get('podSelector')
+
+            i_matched, i_rules = _parse_rules(
+                'ingress', crd, namespace=namespace)
+            e_matched, e_rules = _parse_rules(
+                'egress', crd, namespace=namespace)
+
+            if i_matched or e_matched:
+                driver_utils.patch_kuryr_crd(crd, i_rules,
+                                             e_rules, crd_selector)
+
+    def update_namespace_sg_rules(self, namespace):
+        LOG.debug("Updating sg rule for namespace: %s",
+                  namespace['metadata']['name'])
+        self.delete_namespace_sg_rules(namespace)
+        self.create_namespace_sg_rules(namespace)
 
     def create_namespace_sg(self, namespace, project_id, crd_spec):
         LOG.debug("Security group driver does not create SGs for the "
@@ -220,18 +351,6 @@ class NetworkPolicySecurityGroupsDriver(base.PodSecurityGroupsDriver):
         LOG.debug("Security group driver does not implement deleting "
                   "SGs.")
 
-    def delete_namespace_sg_rules(self, namespace):
-        LOG.debug("Security group driver does not delete SG rules for "
-                  "namespace.")
-
-    def create_namespace_sg_rules(self, namespace):
-        LOG.debug("Security group driver does not create SG rules for "
-                  "namespace.")
-
-    def update_namespace_sg_rules(self, namespace):
-        LOG.debug("Security group driver does not update SG rules for "
-                  "namespace.")
-
 
 class NetworkPolicyServiceSecurityGroupsDriver(
         base.ServiceSecurityGroupsDriver):
