@@ -225,11 +225,11 @@ def patch_kuryr_crd(crd, i_rules, e_rules, pod_selector, np_spec=None):
         np_spec = crd['spec']['networkpolicy_spec']
     LOG.debug('Patching KuryrNetPolicy CRD %s' % crd_name)
     try:
-        kubernetes.patch('spec', crd['metadata']['selfLink'],
-                         {'ingressSgRules': i_rules,
-                          'egressSgRules': e_rules,
-                          'podSelector': pod_selector,
-                          'networkpolicy_spec': np_spec})
+        kubernetes.patch_crd('spec', crd['metadata']['selfLink'],
+                             {'ingressSgRules': i_rules,
+                              'egressSgRules': e_rules,
+                              'podSelector': pod_selector,
+                              'networkpolicy_spec': np_spec})
     except k_exc.K8sClientException:
         LOG.exception('Error updating kuryrnetpolicy CRD %s', crd_name)
         raise
@@ -379,3 +379,47 @@ def get_namespace_subnet_cidr(namespace):
         LOG.exception("Kubernetes Client Exception.")
         raise
     return net_crd['spec']['subnetCIDR']
+
+
+def tag_neutron_resources(resource, res_ids):
+    tags = CONF.neutron_defaults.resource_tags
+    if tags:
+        neutron = clients.get_neutron_client()
+        for res_id in res_ids:
+            try:
+                neutron.replace_tag(resource, res_id, body={"tags": tags})
+            except n_exc.NeutronClientException:
+                LOG.warning("Failed to tag %s %s with %s. Ignoring, but this "
+                            "is still unexpected.", resource, res_id, tags,
+                            exc_info=True)
+
+
+def get_services(namespace):
+    kubernetes = clients.get_kubernetes_client()
+    try:
+        services = kubernetes.get(
+            '{}/namespaces/{}/services'.format(constants.K8S_API_BASE,
+                                               namespace))
+    except k_exc.K8sClientException:
+        LOG.exception('Exception when getting K8s services in '
+                      'namespace %s', namespace)
+        raise
+    return services
+
+
+def service_matches_affected_pods(service, pod_selectors):
+    """Returns if the service is affected by the pod selectors
+
+    Checks if the service selector matches the labelSelectors of
+    NetworkPolicies.
+
+    param service: k8s service
+    param pod_selectors: a list of kubernetes labelSelectors
+    return: True if the service is selected by any of the labelSelectors
+            and False otherwise.
+    """
+    svc_selector = service['spec'].get('selector')
+    for selector in pod_selectors:
+        if match_selector(selector, svc_selector):
+            return True
+    return False
