@@ -24,6 +24,7 @@ from kuryr_kubernetes.controller.drivers import base as drv_base
 from kuryr_kubernetes import exceptions as k_exc
 from kuryr_kubernetes.handlers import k8s_base
 from kuryr_kubernetes.objects import lbaas as obj_lbaas
+from kuryr_kubernetes import utils
 
 LOG = logging.getLogger(__name__)
 
@@ -47,7 +48,7 @@ class LBaaSSpecHandler(k8s_base.ResourceEventHandler):
         self._drv_sg = drv_base.ServiceSecurityGroupsDriver.get_instance()
 
     def on_present(self, service):
-        lbaas_spec = self._get_lbaas_spec(service)
+        lbaas_spec = utils.get_lbaas_spec(service)
 
         if self._should_ignore(service):
             LOG.debug("Skipping Kubernetes service %s of an unsupported kind "
@@ -58,7 +59,7 @@ class LBaaSSpecHandler(k8s_base.ResourceEventHandler):
 
         if self._has_lbaas_spec_changes(service, lbaas_spec):
             lbaas_spec = self._generate_lbaas_spec(service)
-            self._set_lbaas_spec(service, lbaas_spec)
+            utils.set_lbaas_spec(service, lbaas_spec)
 
     def _is_supported_type(self, service):
         spec = service['spec']
@@ -117,29 +118,7 @@ class LBaaSSpecHandler(k8s_base.ResourceEventHandler):
 
     def _has_lbaas_spec_changes(self, service, lbaas_spec):
         return (self._has_ip_changes(service, lbaas_spec) or
-                self._has_port_changes(service, lbaas_spec))
-
-    def _get_service_ports(self, service):
-        return [{'name': port.get('name'),
-                 'protocol': port.get('protocol', 'TCP'),
-                 'port': port['port']}
-                for port in service['spec']['ports']]
-
-    def _has_port_changes(self, service, lbaas_spec):
-        link = service['metadata']['selfLink']
-
-        fields = obj_lbaas.LBaaSPortSpec.fields
-        svc_port_set = {tuple(port[attr] for attr in fields)
-                        for port in self._get_service_ports(service)}
-        spec_port_set = {tuple(getattr(port, attr) for attr in fields)
-                         for port in lbaas_spec.ports}
-
-        if svc_port_set != spec_port_set:
-            LOG.debug("LBaaS spec ports %(spec_ports)s != %(svc_ports)s "
-                      "for %(link)s" % {'spec_ports': spec_port_set,
-                                        'svc_ports': svc_port_set,
-                                        'link': link})
-        return svc_port_set != spec_port_set
+                utils.has_port_changes(service, lbaas_spec))
 
     def _has_ip_changes(self, service, lbaas_spec):
         link = service['metadata']['selfLink']
@@ -161,56 +140,7 @@ class LBaaSSpecHandler(k8s_base.ResourceEventHandler):
 
     def _generate_lbaas_port_specs(self, service):
         return [obj_lbaas.LBaaSPortSpec(**port)
-                for port in self._get_service_ports(service)]
-
-    def _get_endpoints_link(self, service):
-        svc_link = service['metadata']['selfLink']
-        link_parts = svc_link.split('/')
-
-        if link_parts[-2] != 'services':
-            raise k_exc.IntegrityError(_(
-                "Unsupported service link: %(link)s") % {
-                'link': svc_link})
-        link_parts[-2] = 'endpoints'
-
-        return "/".join(link_parts)
-
-    def _set_lbaas_spec(self, service, lbaas_spec):
-        # TODO(ivc): extract annotation interactions
-        if lbaas_spec is None:
-            LOG.debug("Removing LBaaSServiceSpec annotation: %r", lbaas_spec)
-            annotation = None
-        else:
-            lbaas_spec.obj_reset_changes(recursive=True)
-            LOG.debug("Setting LBaaSServiceSpec annotation: %r", lbaas_spec)
-            annotation = jsonutils.dumps(lbaas_spec.obj_to_primitive(),
-                                         sort_keys=True)
-        svc_link = service['metadata']['selfLink']
-        ep_link = self._get_endpoints_link(service)
-        k8s = clients.get_kubernetes_client()
-
-        try:
-            k8s.annotate(ep_link,
-                         {k_const.K8S_ANNOTATION_LBAAS_SPEC: annotation})
-        except k_exc.K8sClientException:
-            # REVISIT(ivc): only raise ResourceNotReady for NotFound
-            raise k_exc.ResourceNotReady(ep_link)
-
-        k8s.annotate(svc_link,
-                     {k_const.K8S_ANNOTATION_LBAAS_SPEC: annotation},
-                     resource_version=service['metadata']['resourceVersion'])
-
-    def _get_lbaas_spec(self, service):
-        # TODO(ivc): same as '_set_lbaas_spec'
-        try:
-            annotations = service['metadata']['annotations']
-            annotation = annotations[k_const.K8S_ANNOTATION_LBAAS_SPEC]
-        except KeyError:
-            return None
-        obj_dict = jsonutils.loads(annotation)
-        obj = obj_lbaas.LBaaSServiceSpec.obj_from_primitive(obj_dict)
-        LOG.debug("Got LBaaSServiceSpec from annotation: %r", obj)
-        return obj
+                for port in utils.get_service_ports(service)]
 
 
 class LoadBalancerHandler(k8s_base.ResourceEventHandler):
@@ -294,30 +224,32 @@ class LoadBalancerHandler(k8s_base.ResourceEventHandler):
         if not lbaas_state:
             return
         # NOTE(ivc): deleting pool deletes its members
-        if self._drv_lbaas.cascading_capable:
-            self._drv_lbaas.release_loadbalancer(
-                loadbalancer=lbaas_state.loadbalancer)
-            if lbaas_state.service_pub_ip_info:
-                self._drv_service_pub_ip.release_pub_ip(
-                    lbaas_state.service_pub_ip_info)
-        else:
-            lbaas_state.members = []
-            self._sync_lbaas_members(endpoints, lbaas_state,
-                                     obj_lbaas.LBaaSServiceSpec())
+        self._drv_lbaas.release_loadbalancer(
+            loadbalancer=lbaas_state.loadbalancer)
+        if lbaas_state.service_pub_ip_info:
+            self._drv_service_pub_ip.release_pub_ip(
+                lbaas_state.service_pub_ip_info)
 
     def _should_ignore(self, endpoints, lbaas_spec):
+        # NOTE(ltomasbo): we must wait until service handler has annotated the
+        # endpoints to process them. Thus, if annotations are not updated to
+        # match the endpoints information, we should skip the event
         return not(lbaas_spec and
                    self._has_pods(endpoints) and
-                   self._is_lbaas_spec_in_sync(endpoints, lbaas_spec))
+                   self._svc_handler_annotations_updated(endpoints,
+                                                         lbaas_spec))
 
-    def _is_lbaas_spec_in_sync(self, endpoints, lbaas_spec):
-        # REVISIT(ivc): consider other options instead of using 'name'
-        ep_ports = list(set(port.get('name')
-                            for subset in endpoints.get('subsets', [])
-                            for port in subset.get('ports', [])))
-        spec_ports = [port.name for port in lbaas_spec.ports]
-
-        return sorted(ep_ports) == sorted(spec_ports)
+    def _svc_handler_annotations_updated(self, endpoints, lbaas_spec):
+        svc_link = self._get_service_link(endpoints)
+        k8s = clients.get_kubernetes_client()
+        service = k8s.get(svc_link)
+        if utils.has_port_changes(service, lbaas_spec):
+            # NOTE(ltomasbo): Ensuring lbaas_spec annotated on the endpoints
+            # is in sync with the service status, i.e., upon a service
+            # modification it will ensure endpoint modifications are not
+            # handled until the service handler has performed its annotations
+            return False
+        return True
 
     def _has_pods(self, endpoints):
         ep_subsets = endpoints.get('subsets', [])
@@ -345,13 +277,35 @@ class LoadBalancerHandler(k8s_base.ResourceEventHandler):
 
         return changed
 
+    def _sync_lbaas_sgs(self, endpoints, lbaas_state, lbaas_spec):
+        # NOTE (maysams) Need to retrieve the LBaaS Spec again due to
+        # the possibility of it being updated after the LBaaS creation
+        # process has started.
+        svc_link = self._get_service_link(endpoints)
+        k8s = clients.get_kubernetes_client()
+        service = k8s.get(svc_link)
+        lbaas_spec = utils.get_lbaas_spec(service)
+
+        lb = lbaas_state.loadbalancer
+        default_sgs = config.CONF.neutron_defaults.pod_security_groups
+        lbaas_spec_sgs = lbaas_spec.security_groups_ids
+        if lb.security_groups and lb.security_groups != lbaas_spec_sgs:
+            sgs = [lb_sg for lb_sg in lb.security_groups
+                   if lb_sg not in default_sgs]
+            if lbaas_spec_sgs != default_sgs:
+                sgs.extend(lbaas_spec_sgs)
+            lb.security_groups = sgs
+
     def _add_new_members(self, endpoints, lbaas_state, lbaas_spec):
         changed = False
 
+        self._sync_lbaas_sgs(endpoints, lbaas_state, lbaas_spec)
+
         lsnr_by_id = {l.id: l for l in lbaas_state.listeners}
         pool_by_lsnr_port = {(lsnr_by_id[p.listener_id].protocol,
                               lsnr_by_id[p.listener_id].port): p
                              for p in lbaas_state.pools}
+
         # NOTE(yboaron): Since LBaaSv2 doesn't support UDP load balancing,
         #              the LBaaS driver will return 'None' in case of UDP port
         #              listener creation.
@@ -387,6 +341,7 @@ class LoadBalancerHandler(k8s_base.ResourceEventHandler):
                     except KeyError:
                         LOG.debug("No pool found for port: %r", port_name)
                         continue
+
                     if (target_ip, target_port, pool.id) in current_targets:
                         continue
                     # TODO(apuimedo): Do not pass subnet_id at all when in
@@ -410,6 +365,7 @@ class LoadBalancerHandler(k8s_base.ResourceEventHandler):
                         listener_port = lsnr_by_id[pool.listener_id].port
                     else:
                         listener_port = None
+
                     member = self._drv_lbaas.ensure_member(
                         loadbalancer=lbaas_state.loadbalancer,
                         pool=pool,
@@ -436,20 +392,38 @@ class LoadBalancerHandler(k8s_base.ResourceEventHandler):
                 for subnet in network.subnets.objects
                 if ip in subnet.cidr][0]
 
+    def _get_port_in_pool(self, pool, lbaas_state, lbaas_spec):
+        for l in lbaas_state.listeners:
+            if l.id != pool.listener_id:
+                continue
+            for port in lbaas_spec.ports:
+                if l.port == port.port and l.protocol == port.protocol:
+                    return port
+        return None
+
     def _remove_unused_members(self, endpoints, lbaas_state, lbaas_spec):
-        spec_port_names = {p.name for p in lbaas_spec.ports}
-        current_targets = {(a['ip'], p['port'])
+        spec_ports = {}
+        for pool in lbaas_state.pools:
+            port = self._get_port_in_pool(pool, lbaas_state, lbaas_spec)
+            if port:
+                spec_ports[port.name] = pool.id
+
+        current_targets = {(a['ip'], p['port'],
+                            spec_ports.get(p.get('name')))
                            for s in endpoints['subsets']
                            for a in s['addresses']
                            for p in s['ports']
-                           if p.get('name') in spec_port_names}
+                           if p.get('name') in spec_ports}
+
         removed_ids = set()
         for member in lbaas_state.members:
-            if (str(member.ip), member.port) in current_targets:
+            if ((str(member.ip), member.port, member.pool_id) in
+                    current_targets):
                 continue
             self._drv_lbaas.release_member(lbaas_state.loadbalancer,
                                            member)
             removed_ids.add(member.id)
+
         if removed_ids:
             lbaas_state.members = [m for m in lbaas_state.members
                                    if m.id not in removed_ids]
