@@ -96,6 +96,7 @@ class TestNetworkPolicyDriver(test_base.TestCase):
 
         self._crd = {
             'metadata': {'name': mock.sentinel.name,
+                         'namespace': u'default',
                          'selfLink': mock.sentinel.selfLink},
             'spec': {
                 'egressSgRules': [
@@ -180,6 +181,8 @@ class TestNetworkPolicyDriver(test_base.TestCase):
         m_affected.assert_not_called()
         m_namespaced.assert_called_once_with(self._policy)
 
+    @mock.patch.object(network_policy.NetworkPolicyDriver,
+                       '_add_default_np_rules')
     @mock.patch.object(network_policy.NetworkPolicyDriver,
                        'get_kuryrnetpolicy_crd')
     @mock.patch.object(network_policy.NetworkPolicyDriver,
@@ -190,7 +193,8 @@ class TestNetworkPolicyDriver(test_base.TestCase):
     def test_create_security_group_rules_from_network_policy(self, m_utils,
                                                              m_parse,
                                                              m_add_crd,
-                                                             m_get_crd):
+                                                             m_get_crd,
+                                                             m_add_default):
         self._driver.neutron.create_security_group.return_value = {
             'security_group': {'id': mock.sentinel.id}}
         m_utils.get_subnet_cidr.return_value = {
@@ -202,7 +206,10 @@ class TestNetworkPolicyDriver(test_base.TestCase):
             self._policy, self._project_id)
         m_get_crd.assert_called_once()
         m_add_crd.assert_called_once()
+        m_add_default.assert_called_once()
 
+    @mock.patch.object(network_policy.NetworkPolicyDriver,
+                       '_add_default_np_rules')
     @mock.patch.object(network_policy.NetworkPolicyDriver,
                        'get_kuryrnetpolicy_crd')
     @mock.patch.object(network_policy.NetworkPolicyDriver,
@@ -211,7 +218,8 @@ class TestNetworkPolicyDriver(test_base.TestCase):
                        'parse_network_policy_rules')
     @mock.patch.object(utils, 'get_subnet_cidr')
     def test_create_security_group_rules_with_k8s_exc(self, m_utils, m_parse,
-                                                      m_add_crd, m_get_crd):
+                                                      m_add_crd, m_get_crd,
+                                                      m_add_default):
         self._driver.neutron.create_security_group.return_value = {
             'security_group': {'id': mock.sentinel.id}}
         m_utils.get_subnet_cidr.return_value = {
@@ -225,7 +233,10 @@ class TestNetworkPolicyDriver(test_base.TestCase):
             self._driver.create_security_group_rules_from_network_policy,
             self._policy, self._project_id)
         m_add_crd.assert_called_once()
+        m_add_default.assert_called_once()
 
+    @mock.patch.object(network_policy.NetworkPolicyDriver,
+                       '_add_default_np_rules')
     @mock.patch.object(network_policy.NetworkPolicyDriver,
                        'get_kuryrnetpolicy_crd')
     @mock.patch.object(network_policy.NetworkPolicyDriver,
@@ -234,7 +245,8 @@ class TestNetworkPolicyDriver(test_base.TestCase):
                        'parse_network_policy_rules')
     @mock.patch.object(utils, 'get_subnet_cidr')
     def test_create_security_group_rules_error_add_crd(self, m_utils, m_parse,
-                                                       m_add_crd, m_get_crd):
+                                                       m_add_crd, m_get_crd,
+                                                       m_add_default):
         self._driver.neutron.create_security_group.return_value = {
             'security_group': {'id': mock.sentinel.id}}
         m_utils.get_subnet_cidr.return_value = {
@@ -248,6 +260,7 @@ class TestNetworkPolicyDriver(test_base.TestCase):
             self._driver.create_security_group_rules_from_network_policy,
             self._policy, self._project_id)
         m_get_crd.assert_not_called()
+        m_add_default.assert_called_once()
 
     def test_create_security_group_rules_with_n_exc(self):
         self._driver.neutron.create_security_group.side_effect = (
@@ -281,7 +294,7 @@ class TestNetworkPolicyDriver(test_base.TestCase):
                        'parse_network_policy_rules')
     def test_update_security_group_rules_with_k8s_exc(self, m_parse, m_get_crd,
                                                       m_create_sgr):
-        self._driver.kubernetes.patch.side_effect = (
+        self._driver.kubernetes.patch_crd.side_effect = (
             exceptions.K8sClientException())
         m_get_crd.return_value = self._crd
         m_parse.return_value = (self._i_rules, self._e_rules)
@@ -428,3 +441,18 @@ class TestNetworkPolicyDriver(test_base.TestCase):
 
         resp = self._driver.namespaced_pods(self._policy)
         self.assertEqual([], resp)
+
+    @mock.patch.object(network_policy.NetworkPolicyDriver,
+                       '_del_kuryrnetpolicy_crd', return_value=False)
+    def test_release_network_policy(self, m_del_crd):
+        self._driver.release_network_policy(self._crd)
+        self.neutron.delete_security_group.assert_called_once_with(
+            self._crd['spec']['securityGroupId'])
+        m_del_crd.assert_called_once_with(self._crd['metadata']['name'],
+                                          self._crd['metadata']['namespace'])
+
+    @mock.patch.object(network_policy.NetworkPolicyDriver,
+                       '_del_kuryrnetpolicy_crd', return_value=False)
+    def test_release_network_policy_removed_crd(self, m_del_crd):
+        self._driver.release_network_policy(None)
+        m_del_crd.assert_not_called()
