@@ -21,6 +21,131 @@ from kuryr_kubernetes.tests.unit import kuryr_fixtures as k_fix
 from oslo_config import cfg
 
 
+def get_no_match_crd_namespace_obj():
+    return {
+        "kind": "Namespace",
+        "metadata": {
+            "annotations": {
+                "openstack.org/kuryr-namespace-label": '{"name": "dev"}',
+                "openstack.org/kuryr-net-crd": "ns-dev"
+            },
+            "labels": {"name": "prod"},
+            "name": "prod",
+            "selfLink": "/api/v1/namespaces/dev"}}
+
+
+def get_match_crd_namespace_obj():
+    return {
+        "kind": "Namespace",
+        "metadata": {
+            "annotations": {
+                "openstack.org/kuryr-namespace-label": '{"name": "dev"}',
+                "openstack.org/kuryr-net-crd": "ns-dev"
+            },
+            "labels": {
+                "name": "dev"
+            },
+            "name": "dev",
+            "selfLink": "/api/v1/namespaces/dev"}}
+
+
+def get_match_crd_pod_obj():
+    return {
+        'kind': 'Pod',
+        'metadata': {
+            'name': mock.sentinel.pod_name,
+            'namespace': 'dev',
+            'labels': {
+                'tier': 'backend'},
+            'annotations': {
+                'openstack.org/kuryr-pod-label': '{"tier": "backend"}'}},
+        'status': {'podIP': mock.sentinel.podIP}}
+
+
+def get_sg_rule():
+    pod_ip = get_match_crd_pod_obj()['status'].get('podIP')
+    return {
+        "namespace": 'dev',
+        "security_group_rule": {
+            "description": "Kuryr-Kubernetes NetPolicy SG rule",
+            "direction": "ingress",
+            "ethertype": "IPv4",
+            "id": 'f15ff50a-e8a4-4872-81bf-a04cbb8cb388',
+            "port_range_max": 6379,
+            "port_range_min": 6379,
+            "protocol": "tcp",
+            "remote_ip_prefix": pod_ip,
+            "security_group_id": '36923e76-026c-422b-8dfd-7292e7c88228'}}
+
+
+def get_matched_crd_obj():
+    return {
+        "kind": "KuryrNetPolicy",
+        "metadata": {"name": "np-test-network-policy",
+                     "namespace": "default"},
+        "spec": {
+            "egressSgRules": [],
+            "ingressSgRules": [get_sg_rule()],
+            "networkpolicy_spec": {
+                "ingress": [
+                    {"from": [
+                        {"namespaceSelector": {
+                            "matchLabels": {"name": "dev"}}}],
+                     "ports": [
+                        {"port": 6379,
+                         "protocol": "TCP"}]}],
+                "podSelector": {"matchLabels": {"app": "demo"}},
+                "policyTypes": ["Ingress"]},
+            "podSelector": {"matchLabels": {"app": "demo"}},
+            "securityGroupId": '36923e76-026c-422b-8dfd-7292e7c88228'}}
+
+
+def get_crd_obj_no_match():
+    return {
+        "kind": "KuryrNetPolicy",
+        "metadata": {"name": "np-test-network-policy",
+                     "namespace": "default"},
+        "spec": {
+            "egressSgRules": [],
+            "ingressSgRules": [],
+            "networkpolicy_spec": {
+                "ingress": [
+                    {"from": [
+                        {"namespaceSelector": {
+                            "matchLabels": {"name": "dev"}}}],
+                     "ports": [
+                        {"port": 6379,
+                         "protocol": "TCP"}]}],
+                "podSelector": {"matchLabels": {"app": "demo"}},
+                "policyTypes": ["Ingress"]},
+            "podSelector": {"matchLabels": {"app": "demo"}},
+            "securityGroupId": '36923e76-026c-422b-8dfd-7292e7c88228'}}
+
+
+def get_crd_obj_with_all_selectors():
+    return {
+        "kind": "KuryrNetPolicy",
+        "metadata": {"name": "np-test-network-policy",
+                     "namespace": "default"},
+        "spec": {
+            "egressSgRules": [],
+            "ingressSgRules": [],
+            "networkpolicy_spec": {
+                "ingress": [
+                    {"from": [
+                        {"namespaceSelector": {
+                            "matchLabels": {"name": "dev"}},
+                         "podSelector": {
+                            "matchLabels": {"tier": "backend"}}}],
+                     "ports": [
+                        {"port": 6379,
+                         "protocol": "TCP"}]}],
+                "podSelector": {"matchLabels": {"app": "demo"}},
+                "policyTypes": ["Ingress"]},
+            "podSelector": {"matchLabels": {"app": "demo"}},
+            "securityGroupId": '36923e76-026c-422b-8dfd-7292e7c88228'}}
+
+
 class TestNetworkPolicySecurityGroupsDriver(test_base.TestCase):
 
     def setUp(self):
@@ -103,14 +228,6 @@ class TestNetworkPolicySecurityGroupsDriver(test_base.TestCase):
                 "resourceVersion": "",
                 "selfLink": mock.sentinel.selfLink}}
 
-        self._empty_crds = {
-            "apiVersion": "v1",
-            "items": [],
-            "kind": "List",
-            "metadata": {
-                "resourceVersion": "",
-                "selfLink": mock.sentinel.selfLink}}
-
         self._pod = {
             'apiVersion': 'v1',
             'kind': 'Pod',
@@ -166,30 +283,6 @@ class TestNetworkPolicySecurityGroupsDriver(test_base.TestCase):
         self._driver = (
             network_policy_security_groups.NetworkPolicySecurityGroupsDriver())
 
-        self._crd_sg_id = mock.sentinel.crd_sg_id
-        self._crd_without_rules = {
-            "apiVersion": "openstack.org/v1",
-            "kind": "KuryrNetPolicy",
-            "metadata": {"name": "np-test-network-policy",
-                          "namespace": "default"},
-            "spec": {
-                "egressSgRules": [],
-                "ingressSgRules": [],
-                "networkpolicy_spec": {
-                    "ingress": [
-                        {"from": [
-                            {"namespaceSelector": {
-                                "matchLabels": {"name": "dev"}},
-                             "podSelector": {
-                                "matchLabels": {"tier": "backend"}}}],
-                         "ports": [
-                            {"port": 6379,
-                             "protocol": "TCP"}]}],
-                    "podSelector": {"matchLabels": {"app": "demo"}},
-                    "policyTypes": ["Ingress"]},
-                "podSelector": {"matchLabels": {"app": "demo"}},
-                "securityGroupId": self._crd_sg_id}}
-
         self._pod_ip = mock.sentinel.pod_ip
         self._pod_dev_namespace = {
             'apiVersion': 'v1',
@@ -209,6 +302,7 @@ class TestNetworkPolicySecurityGroupsDriver(test_base.TestCase):
                     }]},
             'status': {'podIP': self._pod_ip}}
 
+        self._crd_sg_id = mock.sentinel.crd_sg_id
         self._sg_rule_body = {
             u'security_group_rule': {
                 u'direction': 'ingress',
@@ -265,12 +359,12 @@ class TestNetworkPolicySecurityGroupsDriver(test_base.TestCase):
                               m_match_selector,
                               m_create_sg_rule_body,
                               m_create_sg_rule):
-        m_get_pod_ip.return_value = self._pod_ip
         m_create_sg_rule_body.return_value = self._sg_rule_body
         sgr_id = mock.sentinel.sgr_id
         m_create_sg_rule.return_value = sgr_id
-        crd = self._crd_without_rules
-        pod = self._pod_dev_namespace
+        crd = get_crd_obj_with_all_selectors()
+        pod = get_match_crd_pod_obj()
+        m_get_pod_ip.return_value = pod['status'].get('podIP')
         matched = False
         new_sg_rule = self._sg_rule_body
 
@@ -298,7 +392,7 @@ class TestNetworkPolicySecurityGroupsDriver(test_base.TestCase):
     @mock.patch('kuryr_kubernetes.controller.drivers.utils.'
                 'match_selector', return_value=False)
     def test__create_sg_rules_no_match(self, m_match_selector):
-        crd = self._crd_without_rules
+        crd = get_crd_obj_with_all_selectors()
         pod = self._pod2
 
         policy = crd['spec']['networkpolicy_spec']
@@ -408,7 +502,7 @@ class TestNetworkPolicySecurityGroupsDriver(test_base.TestCase):
     @mock.patch('kuryr_kubernetes.controller.drivers.utils.'
                 'get_kuryrnetpolicy_crds')
     def test_get_sgs_no_crds(self, m_get_crds):
-        m_get_crds.return_value = self._empty_crds
+        m_get_crds.return_value = {"items": []}
         cfg.CONF.set_override('pod_security_groups', [],
                               group='neutron_defaults')
 
@@ -433,3 +527,134 @@ class TestNetworkPolicySecurityGroupsDriver(test_base.TestCase):
 
         m_get_crds.assert_called_once_with(namespace=self._namespace)
         self.assertEqual([str(self._sg_id), str(self._sg_id2)], resp)
+
+    @mock.patch('kuryr_kubernetes.controller.drivers.utils.'
+                'patch_kuryr_crd')
+    @mock.patch('kuryr_kubernetes.controller.drivers.utils.'
+                'delete_security_group_rule')
+    @mock.patch('kuryr_kubernetes.controller.drivers.utils.'
+                'get_kuryrnetpolicy_crds')
+    def test_delete_namespace_sg_rule(self, m_get_knp_crd, m_delete_sg_rule,
+                                      m_patch_kuryr_crd):
+        cls = network_policy_security_groups.NetworkPolicySecurityGroupsDriver
+        m_driver = mock.MagicMock(spec=cls)
+        i_rule = get_matched_crd_obj()['spec']['ingressSgRules'][0]
+        sg_rule_id = i_rule.get('security_group_rule')['id']
+
+        m_get_knp_crd.return_value = {"items": [get_matched_crd_obj()]}
+
+        cls.delete_namespace_sg_rules(m_driver, get_match_crd_namespace_obj())
+
+        m_get_knp_crd.assert_called_once()
+        m_delete_sg_rule.assert_called_once_with(sg_rule_id)
+        m_patch_kuryr_crd.assert_called_once()
+
+    @mock.patch('kuryr_kubernetes.controller.drivers.utils.'
+                'patch_kuryr_crd')
+    @mock.patch('kuryr_kubernetes.controller.drivers.utils.'
+                'delete_security_group_rule')
+    @mock.patch('kuryr_kubernetes.controller.drivers.utils.'
+                'get_kuryrnetpolicy_crds')
+    def test_delete_namespace_sg_rule_no_match(self, m_get_knp_crd,
+                                               m_delete_sg_rule,
+                                               m_patch_kuryr_crd):
+        cls = network_policy_security_groups.NetworkPolicySecurityGroupsDriver
+        m_driver = mock.MagicMock(spec=cls)
+
+        m_get_knp_crd.return_value = {"items": [get_matched_crd_obj()]}
+
+        cls.delete_namespace_sg_rules(m_driver,
+                                      get_no_match_crd_namespace_obj())
+
+        m_get_knp_crd.assert_called_once()
+        m_delete_sg_rule.assert_not_called()
+        m_patch_kuryr_crd.assert_not_called()
+
+    @mock.patch('kuryr_kubernetes.controller.drivers.'
+                'network_policy_security_groups._create_sg_rule')
+    @mock.patch('kuryr_kubernetes.controller.drivers.utils.'
+                'match_selector')
+    @mock.patch('kuryr_kubernetes.controller.drivers.utils.'
+                'get_namespace_subnet_cidr')
+    def test__parse_rules(self, m_get_ns_subnet_cidr, m_match_selector,
+                          m_create_sg_rule):
+        crd = get_crd_obj_no_match()
+        policy = crd['spec']['networkpolicy_spec']
+        i_rule = policy.get('ingress')[0]
+        ns_selector = i_rule['from'][0].get('namespaceSelector')
+        ns = get_match_crd_namespace_obj()
+
+        m_get_ns_subnet_cidr.return_value = '10.0.2.0/26'
+        m_match_selector.return_value = True
+        m_create_sg_rule.return_value = get_sg_rule()
+
+        matched, rules = network_policy_security_groups._parse_rules(
+            'ingress', crd, namespace=ns)
+
+        m_get_ns_subnet_cidr.assert_called_once_with(ns)
+        m_match_selector.assert_called_once_with(ns_selector,
+                                                 ns['metadata']['labels'])
+        m_create_sg_rule.assert_called_once()
+
+        self.assertEqual(matched, True)
+        self.assertEqual(rules, [get_sg_rule()])
+
+    @mock.patch('kuryr_kubernetes.controller.drivers.'
+                'network_policy_security_groups._create_sg_rule')
+    @mock.patch('kuryr_kubernetes.controller.drivers.utils.'
+                'match_selector')
+    def test__parse_rules_no_match(self, m_match_selector,
+                                   m_create_sg_rule):
+        crd = get_crd_obj_no_match()
+        policy = crd['spec']['networkpolicy_spec']
+        i_rule = policy.get('ingress')[0]
+        ns_selector = i_rule['from'][0].get('namespaceSelector')
+        ns = get_no_match_crd_namespace_obj()
+
+        m_match_selector.return_value = False
+
+        matched, rules = network_policy_security_groups._parse_rules(
+            'ingress', crd, namespace=ns)
+
+        m_match_selector.assert_called_once_with(ns_selector,
+                                                 ns['metadata']['labels'])
+        m_create_sg_rule.assert_not_called()
+
+        self.assertEqual(matched, False)
+        self.assertEqual(rules, [])
+
+    @mock.patch('kuryr_kubernetes.controller.drivers.'
+                'network_policy_security_groups._create_sg_rule')
+    @mock.patch('kuryr_kubernetes.controller.drivers.utils.'
+                'get_pod_ip')
+    @mock.patch('kuryr_kubernetes.controller.drivers.utils.'
+                'get_pods')
+    @mock.patch('kuryr_kubernetes.controller.drivers.utils.'
+                'match_selector')
+    def test__parse_rules_all_selectors(self, m_match_selector, m_get_pods,
+                                        m_get_pod_ip, m_create_sg_rule):
+        crd = get_crd_obj_with_all_selectors()
+        policy = crd['spec']['networkpolicy_spec']
+        i_rule = policy.get('ingress')[0]
+        ns_selector = i_rule['from'][0].get('namespaceSelector')
+        pod_selector = i_rule['from'][0].get('podSelector')
+        ns = get_match_crd_namespace_obj()
+        pod = get_match_crd_pod_obj()
+
+        m_match_selector.return_value = True
+        m_get_pods.return_value = {"items": [pod]}
+        m_get_pod_ip.return_value = pod['status']['podIP']
+        m_create_sg_rule.return_value = get_sg_rule()
+
+        matched, rules = network_policy_security_groups._parse_rules(
+            'ingress', crd, namespace=ns)
+
+        m_match_selector.assert_called_once_with(ns_selector,
+                                                 ns['metadata']['labels'])
+        m_get_pods.assert_called_once_with(pod_selector,
+                                           ns['metadata']['name'])
+        m_get_pod_ip.assert_called_once_with(pod)
+        m_create_sg_rule.assert_called_once()
+
+        self.assertEqual(matched, True)
+        self.assertEqual(rules, [get_sg_rule()])
