@@ -48,7 +48,9 @@ class TestLBaaSSpecHandler(test_base.TestCase):
         self.assertEqual(mock.sentinel.drv_subnets, handler._drv_subnets)
         self.assertEqual(mock.sentinel.drv_sg, handler._drv_sg)
 
-    def test_on_present(self):
+    @mock.patch('kuryr_kubernetes.utils.set_lbaas_spec')
+    @mock.patch('kuryr_kubernetes.utils.get_lbaas_spec')
+    def test_on_present(self, m_get_lbaas_spec, m_set_lbaas_spec):
         svc_event = mock.sentinel.svc_event
         old_spec = mock.sentinel.old_spec
         new_spec = mock.sentinel.new_spec
@@ -58,7 +60,7 @@ class TestLBaaSSpecHandler(test_base.TestCase):
         m_drv_project.get_project.return_value = project_id
 
         m_handler = mock.Mock(spec=h_lbaas.LBaaSSpecHandler)
-        m_handler._get_lbaas_spec.return_value = old_spec
+        m_get_lbaas_spec.return_value = old_spec
         m_handler._has_lbaas_spec_changes.return_value = True
         m_handler._generate_lbaas_spec.return_value = new_spec
         m_handler._should_ignore.return_value = False
@@ -66,43 +68,49 @@ class TestLBaaSSpecHandler(test_base.TestCase):
 
         h_lbaas.LBaaSSpecHandler.on_present(m_handler, svc_event)
 
-        m_handler._get_lbaas_spec.assert_called_once_with(svc_event)
+        m_get_lbaas_spec.assert_called_once_with(svc_event)
         m_handler._has_lbaas_spec_changes.assert_called_once_with(svc_event,
                                                                   old_spec)
         m_handler._generate_lbaas_spec.assert_called_once_with(svc_event)
-        m_handler._set_lbaas_spec.assert_called_once_with(svc_event, new_spec)
+        m_set_lbaas_spec.assert_called_once_with(svc_event, new_spec)
 
-    def test_on_present_no_changes(self):
+    @mock.patch('kuryr_kubernetes.utils.set_lbaas_spec')
+    @mock.patch('kuryr_kubernetes.utils.get_lbaas_spec')
+    def test_on_present_no_changes(self, m_get_lbaas_spec,
+                                   m_set_lbaas_spec):
         svc_event = mock.sentinel.svc_event
         old_spec = mock.sentinel.old_spec
 
         m_handler = mock.Mock(spec=h_lbaas.LBaaSSpecHandler)
-        m_handler._get_lbaas_spec.return_value = old_spec
+        m_get_lbaas_spec.return_value = old_spec
         m_handler._has_lbaas_spec_changes.return_value = False
         m_handler._should_ignore.return_value = False
 
         h_lbaas.LBaaSSpecHandler.on_present(m_handler, svc_event)
 
-        m_handler._get_lbaas_spec.assert_called_once_with(svc_event)
+        m_get_lbaas_spec.assert_called_once_with(svc_event)
         m_handler._has_lbaas_spec_changes.assert_called_once_with(svc_event,
                                                                   old_spec)
         m_handler._generate_lbaas_spec.assert_not_called()
-        m_handler._set_lbaas_spec.assert_not_called()
+        m_set_lbaas_spec.assert_not_called()
 
-    def test_on_present_no_selector(self):
+    @mock.patch('kuryr_kubernetes.utils.set_lbaas_spec')
+    @mock.patch('kuryr_kubernetes.utils.get_lbaas_spec')
+    def test_on_present_no_selector(self, m_get_lbaas_spec,
+                                    m_set_lbaas_spec):
         svc_event = {'metadata': {'name': 'dummy_name'}}
         old_spec = mock.sentinel.old_spec
 
         m_handler = mock.Mock(spec=h_lbaas.LBaaSSpecHandler)
-        m_handler._get_lbaas_spec.return_value = old_spec
+        m_get_lbaas_spec.return_value = old_spec
         m_handler._should_ignore.return_value = True
 
         h_lbaas.LBaaSSpecHandler.on_present(m_handler, svc_event)
 
-        m_handler._get_lbaas_spec.assert_called_once_with(svc_event)
+        m_get_lbaas_spec.assert_called_once_with(svc_event)
         m_handler._has_lbaas_spec_changes.assert_not_called()
         m_handler._generate_lbaas_spec.assert_not_called()
-        m_handler._set_lbaas_spec.assert_not_called()
+        m_set_lbaas_spec.assert_not_called()
 
     def test_get_service_ip(self):
         svc_body = {'spec': {'type': 'ClusterIP',
@@ -185,7 +193,8 @@ class TestLBaaSSpecHandler(test_base.TestCase):
         m_drv_sg.get_security_groups.assert_called_once_with(
             service, project_id)
 
-    def test_has_lbaas_spec_changes(self):
+    @mock.patch('kuryr_kubernetes.utils.has_port_changes')
+    def test_has_lbaas_spec_changes(self, m_port_changes):
         m_handler = mock.Mock(spec=h_lbaas.LBaaSSpecHandler)
         service = mock.sentinel.service
         lbaas_spec = mock.sentinel.lbaas_spec
@@ -193,61 +202,11 @@ class TestLBaaSSpecHandler(test_base.TestCase):
         for has_ip_changes in (True, False):
             for has_port_changes in (True, False):
                 m_handler._has_ip_changes.return_value = has_ip_changes
-                m_handler._has_port_changes.return_value = has_port_changes
+                m_port_changes.return_value = has_port_changes
                 ret = h_lbaas.LBaaSSpecHandler._has_lbaas_spec_changes(
                     m_handler, service, lbaas_spec)
                 self.assertEqual(has_ip_changes or has_port_changes, ret)
 
-    def test_get_service_ports(self):
-        m_handler = mock.Mock(spec=h_lbaas.LBaaSSpecHandler)
-        service = {'spec': {'ports': [
-            {'port': 1},
-            {'port': 2, 'name': 'X', 'protocol': 'UDP'}
-        ]}}
-        expected_ret = [
-            {'port': 1, 'name': None, 'protocol': 'TCP'},
-            {'port': 2, 'name': 'X', 'protocol': 'UDP'}]
-
-        ret = h_lbaas.LBaaSSpecHandler._get_service_ports(m_handler, service)
-        self.assertEqual(expected_ret, ret)
-
-    def test_has_port_changes(self):
-        m_handler = mock.Mock(spec=h_lbaas.LBaaSSpecHandler)
-        m_service = mock.MagicMock()
-        m_handler._get_service_ports.return_value = [
-            {'port': 1, 'name': 'X', 'protocol': 'TCP'},
-        ]
-
-        m_lbaas_spec = mock.MagicMock()
-        m_lbaas_spec.ports = [
-            obj_lbaas.LBaaSPortSpec(name='X', protocol='TCP', port=1),
-            obj_lbaas.LBaaSPortSpec(name='Y', protocol='TCP', port=2),
-        ]
-
-        ret = h_lbaas.LBaaSSpecHandler._has_port_changes(
-            m_handler, m_service, m_lbaas_spec)
-
-        self.assertTrue(ret)
-
-    def test_has_port_changes__no_changes(self):
-        m_handler = mock.Mock(spec=h_lbaas.LBaaSSpecHandler)
-        m_service = mock.MagicMock()
-        m_handler._get_service_ports.return_value = [
-            {'port': 1, 'name': 'X', 'protocol': 'TCP'},
-            {'port': 2, 'name': 'Y', 'protocol': 'TCP'}
-        ]
-
-        m_lbaas_spec = mock.MagicMock()
-        m_lbaas_spec.ports = [
-            obj_lbaas.LBaaSPortSpec(name='X', protocol='TCP', port=1),
-            obj_lbaas.LBaaSPortSpec(name='Y', protocol='TCP', port=2),
-        ]
-
-        ret = h_lbaas.LBaaSSpecHandler._has_port_changes(
-            m_handler, m_service, m_lbaas_spec)
-
-        self.assertFalse(ret)
-
     def test_has_ip_changes(self):
         m_handler = mock.Mock(spec=h_lbaas.LBaaSSpecHandler)
         m_service = mock.MagicMock()
@@ -290,9 +249,10 @@ class TestLBaaSSpecHandler(test_base.TestCase):
             m_handler, m_service, m_lbaas_spec)
         self.assertFalse(ret)
 
-    def test_generate_lbaas_port_specs(self):
+    @mock.patch('kuryr_kubernetes.utils.get_service_ports')
+    def test_generate_lbaas_port_specs(self, m_get_service_ports):
         m_handler = mock.Mock(spec=h_lbaas.LBaaSSpecHandler)
-        m_handler._get_service_ports.return_value = [
+        m_get_service_ports.return_value = [
             {'port': 1, 'name': 'X', 'protocol': 'TCP'},
             {'port': 2, 'name': 'Y', 'protocol': 'TCP'}
         ]
@@ -304,12 +264,13 @@ class TestLBaaSSpecHandler(test_base.TestCase):
         ret = h_lbaas.LBaaSSpecHandler._generate_lbaas_port_specs(
             m_handler, mock.sentinel.service)
         self.assertEqual(expected_ports, ret)
-        m_handler._get_service_ports.assert_called_once_with(
+        m_get_service_ports.assert_called_once_with(
             mock.sentinel.service)
 
-    def test_generate_lbaas_port_specs_udp(self):
+    @mock.patch('kuryr_kubernetes.utils.get_service_ports')
+    def test_generate_lbaas_port_specs_udp(self, m_get_service_ports):
         m_handler = mock.Mock(spec=h_lbaas.LBaaSSpecHandler)
-        m_handler._get_service_ports.return_value = [
+        m_get_service_ports.return_value = [
             {'port': 1, 'name': 'X', 'protocol': 'TCP'},
             {'port': 2, 'name': 'Y', 'protocol': 'UDP'}
         ]
@@ -321,25 +282,9 @@ class TestLBaaSSpecHandler(test_base.TestCase):
         ret = h_lbaas.LBaaSSpecHandler._generate_lbaas_port_specs(
             m_handler, mock.sentinel.service)
         self.assertEqual(expected_ports, ret)
-        m_handler._get_service_ports.assert_called_once_with(
+        m_get_service_ports.assert_called_once_with(
             mock.sentinel.service)
 
-    def test_get_endpoints_link(self):
-        m_handler = mock.Mock(spec=h_lbaas.LBaaSSpecHandler)
-        service = {'metadata': {
-            'selfLink': "/api/v1/namespaces/default/services/test"}}
-        ret = h_lbaas.LBaaSSpecHandler._get_endpoints_link(m_handler, service)
-        expected_link = "/api/v1/namespaces/default/endpoints/test"
-        self.assertEqual(expected_link, ret)
-
-    def test_get_endpoints_link__integrity_error(self):
-        m_handler = mock.Mock(spec=h_lbaas.LBaaSSpecHandler)
-        service = {'metadata': {
-            'selfLink': "/api/v1/namespaces/default/not-services/test"}}
-        self.assertRaises(k_exc.IntegrityError,
-                          h_lbaas.LBaaSSpecHandler._get_endpoints_link,
-                          m_handler, service)
-
     def test_set_lbaas_spec(self):
         self.skipTest("skipping until generalised annotation handling is "
                       "implemented")
@@ -612,25 +557,6 @@ class TestLoadBalancerHandler(test_base.TestCase):
         m_handler.on_deleted.assert_called_once_with(
             endpoints, lbaas_state)
 
-    @mock.patch('kuryr_kubernetes.objects.lbaas'
-                '.LBaaSServiceSpec')
-    def test_on_deleted(self, m_svc_spec_ctor):
-        endpoints = mock.sentinel.endpoints
-        empty_spec = mock.sentinel.empty_spec
-        lbaas_state = mock.sentinel.lbaas_state
-        m_svc_spec_ctor.return_value = empty_spec
-
-        m_handler = mock.Mock(spec=h_lbaas.LoadBalancerHandler)
-        m_handler._get_lbaas_state.return_value = lbaas_state
-        m_handler._drv_lbaas = mock.Mock()
-        m_handler._drv_lbaas.cascading_capable = False
-
-        h_lbaas.LoadBalancerHandler.on_deleted(m_handler, endpoints)
-
-        m_handler._get_lbaas_state.assert_called_once_with(endpoints)
-        m_handler._sync_lbaas_members.assert_called_once_with(
-            endpoints, lbaas_state, empty_spec)
-
     @mock.patch('kuryr_kubernetes.objects.lbaas'
                 '.LBaaSServiceSpec')
     def test_on_cascade_deleted_lb_service(self, m_svc_spec_ctor):
@@ -645,7 +571,6 @@ class TestLoadBalancerHandler(test_base.TestCase):
         m_handler._get_lbaas_state.return_value = lbaas_state
         m_handler._drv_lbaas = mock.Mock()
         m_handler._drv_service_pub_ip = mock.Mock()
-        m_handler._drv_lbaas.cascading_capable = True
 
         h_lbaas.LoadBalancerHandler.on_deleted(m_handler, endpoints)
 
@@ -661,28 +586,16 @@ class TestLoadBalancerHandler(test_base.TestCase):
         # REVISIT(ivc): ddt?
         m_handler = mock.Mock(spec=h_lbaas.LoadBalancerHandler)
         m_handler._has_pods.return_value = True
-        m_handler._is_lbaas_spec_in_sync.return_value = True
+        m_handler._svc_handler_annotations_updated.return_value = True
 
         ret = h_lbaas.LoadBalancerHandler._should_ignore(
             m_handler, endpoints, lbaas_spec)
         self.assertEqual(False, ret)
 
         m_handler._has_pods.assert_called_once_with(endpoints)
-        m_handler._is_lbaas_spec_in_sync.assert_called_once_with(
+        m_handler._svc_handler_annotations_updated.assert_called_once_with(
             endpoints, lbaas_spec)
 
-    def test_is_lbaas_spec_in_sync(self):
-        names = ['a', 'b', 'c']
-        endpoints = {'subsets': [{'ports': [{'name': n} for n in names]}]}
-        lbaas_spec = obj_lbaas.LBaaSServiceSpec(ports=[
-            obj_lbaas.LBaaSPortSpec(name=n) for n in reversed(names)])
-
-        m_handler = mock.Mock(spec=h_lbaas.LoadBalancerHandler)
-        ret = h_lbaas.LoadBalancerHandler._is_lbaas_spec_in_sync(
-            m_handler, endpoints, lbaas_spec)
-
-        self.assertEqual(True, ret)
-
     def test_has_pods(self):
         # REVISIT(ivc): ddt?
         endpoints = {'subsets': [
@@ -748,10 +661,11 @@ class TestLoadBalancerHandler(test_base.TestCase):
             ip=vip,
             project_id=project_id,
             subnet_id=subnet_id,
-            ports=[obj_lbaas.LBaaSPortSpec(name=str(port),
+            ports=[obj_lbaas.LBaaSPortSpec(name=str(t[0]),
                                            protocol=prot,
-                                           port=port)
-                   for port in set(t[0] for t in targets.values())],
+                                           port=t[0],
+                                           targetPort=t[1])
+                   for t in targets.values()],
             type=lbaas_type)
 
     def _generate_endpoints(self, targets):
@@ -814,6 +728,8 @@ class TestLoadBalancerHandler(test_base.TestCase):
             for member in state.members)
         return observed_targets
 
+    @mock.patch('kuryr_kubernetes.controller.handlers.lbaas.'
+                'LoadBalancerHandler._sync_lbaas_sgs')
     @mock.patch('kuryr_kubernetes.controller.drivers.base'
                 '.PodSubnetsDriver.get_instance')
     @mock.patch('kuryr_kubernetes.controller.drivers.base'
@@ -821,7 +737,7 @@ class TestLoadBalancerHandler(test_base.TestCase):
     @mock.patch('kuryr_kubernetes.controller.drivers.base'
                 '.LBaaSDriver.get_instance')
     def test_sync_lbaas_members(self, m_get_drv_lbaas, m_get_drv_project,
-                                m_get_drv_subnets):
+                                m_get_drv_subnets, m_sync_lbaas_sgs):
         # REVISIT(ivc): test methods separately and verify ensure/release
         project_id = str(uuid.uuid4())
         subnet_id = str(uuid.uuid4())
@@ -848,6 +764,8 @@ class TestLoadBalancerHandler(test_base.TestCase):
         self.assertEqual(sorted(expected_targets.items()), observed_targets)
         self.assertEqual(expected_ip, str(state.loadbalancer.ip))
 
+    @mock.patch('kuryr_kubernetes.controller.handlers.lbaas.'
+                'LoadBalancerHandler._sync_lbaas_sgs')
     @mock.patch('kuryr_kubernetes.controller.drivers.base'
                 '.PodSubnetsDriver.get_instance')
     @mock.patch('kuryr_kubernetes.controller.drivers.base'
@@ -855,7 +773,8 @@ class TestLoadBalancerHandler(test_base.TestCase):
     @mock.patch('kuryr_kubernetes.controller.drivers.base'
                 '.LBaaSDriver.get_instance')
     def test_sync_lbaas_members_udp(self, m_get_drv_lbaas,
-                                    m_get_drv_project, m_get_drv_subnets):
+                                    m_get_drv_project, m_get_drv_subnets,
+                                    m_sync_lbaas_sgs):
         # REVISIT(ivc): test methods separately and verify ensure/release
         project_id = str(uuid.uuid4())
         subnet_id = str(uuid.uuid4())
@@ -882,6 +801,8 @@ class TestLoadBalancerHandler(test_base.TestCase):
         self.assertEqual([], observed_targets)
         self.assertEqual(expected_ip, str(state.loadbalancer.ip))
 
+    @mock.patch('kuryr_kubernetes.controller.handlers.lbaas.'
+                'LoadBalancerHandler._sync_lbaas_sgs')
     @mock.patch('kuryr_kubernetes.controller.drivers.base'
                 '.PodSubnetsDriver.get_instance')
     @mock.patch('kuryr_kubernetes.controller.drivers.base'
@@ -889,7 +810,8 @@ class TestLoadBalancerHandler(test_base.TestCase):
     @mock.patch('kuryr_kubernetes.controller.drivers.base'
                 '.LBaaSDriver.get_instance')
     def test_sync_lbaas_members_svc_listener_port_edit(
-            self, m_get_drv_lbaas, m_get_drv_project, m_get_drv_subnets):
+            self, m_get_drv_lbaas, m_get_drv_project, m_get_drv_subnets,
+            m_sync_lbaas_sgs):
         # REVISIT(ivc): test methods separately and verify ensure/release
         project_id = str(uuid.uuid4())
         subnet_id = str(uuid.uuid4())
@@ -936,6 +858,8 @@ class TestLoadBalancerHandler(test_base.TestCase):
         self.skipTest("skipping until generalised annotation handling is "
                       "implemented")
 
+    @mock.patch('kuryr_kubernetes.controller.handlers.lbaas.'
+                'LoadBalancerHandler._sync_lbaas_sgs')
     @mock.patch('kuryr_kubernetes.controller.drivers.base'
                 '.PodSubnetsDriver.get_instance')
     @mock.patch('kuryr_kubernetes.controller.drivers.base'
@@ -943,7 +867,8 @@ class TestLoadBalancerHandler(test_base.TestCase):
     @mock.patch('kuryr_kubernetes.controller.drivers.base'
                 '.LBaaSDriver.get_instance')
     def test_add_new_members_udp(self, m_get_drv_lbaas,
-                                 m_get_drv_project, m_get_drv_subnets):
+                                 m_get_drv_project, m_get_drv_subnets,
+                                 m_sync_lbaas_sgs):
         project_id = str(uuid.uuid4())
         subnet_id = str(uuid.uuid4())
         current_ip = '1.1.1.1'
