@@ -24,11 +24,12 @@ from oslo_log import log
 from oslo_serialization import jsonutils
 
 from kuryr_kubernetes import clients
+from kuryr_kubernetes import constants
 from kuryr_kubernetes import exceptions
+from kuryr_kubernetes.objects import lbaas as obj_lbaas
 from kuryr_kubernetes.objects import vif
 from kuryr_kubernetes import os_vif_util
 
-
 CONF = cfg.CONF
 LOG = log.getLogger(__name__)
 
@@ -210,3 +211,83 @@ def has_kuryr_crd(crd_url):
                       " CRD. %s" % exceptions.K8sClientException)
         return False
     return True
+
+
+def get_lbaas_spec(service):
+    try:
+        annotations = service['metadata']['annotations']
+        annotation = annotations[constants.K8S_ANNOTATION_LBAAS_SPEC]
+    except KeyError:
+        return None
+    obj_dict = jsonutils.loads(annotation)
+    obj = obj_lbaas.LBaaSServiceSpec.obj_from_primitive(obj_dict)
+    LOG.debug("Got LBaaSServiceSpec from annotation: %r", obj)
+    return obj
+
+
+def set_lbaas_spec(service, lbaas_spec):
+    # TODO(ivc): extract annotation interactions
+    if lbaas_spec is None:
+        LOG.debug("Removing LBaaSServiceSpec annotation: %r", lbaas_spec)
+        annotation = None
+    else:
+        lbaas_spec.obj_reset_changes(recursive=True)
+        LOG.debug("Setting LBaaSServiceSpec annotation: %r", lbaas_spec)
+        annotation = jsonutils.dumps(lbaas_spec.obj_to_primitive(),
+                                     sort_keys=True)
+    svc_link = service['metadata']['selfLink']
+    ep_link = get_endpoints_link(service)
+    k8s = clients.get_kubernetes_client()
+
+    try:
+        k8s.annotate(ep_link,
+                     {constants.K8S_ANNOTATION_LBAAS_SPEC: annotation})
+    except exceptions.K8sResourceNotFound:
+        raise exceptions.ResourceNotReady(ep_link)
+    except exceptions.K8sClientException:
+        LOG.debug("Failed to annotate endpoint %r", ep_link)
+        raise
+    k8s.annotate(svc_link,
+                 {constants.K8S_ANNOTATION_LBAAS_SPEC: annotation},
+                 resource_version=service['metadata']['resourceVersion'])
+
+
+def get_endpoints_link(service):
+    svc_link = service['metadata']['selfLink']
+    link_parts = svc_link.split('/')
+
+    if link_parts[-2] != 'services':
+        raise exceptions.IntegrityError(_(
+            "Unsupported service link: %(link)s") % {
+            'link': svc_link})
+    link_parts[-2] = 'endpoints'
+
+    return "/".join(link_parts)
+
+
+def has_port_changes(service, lbaas_spec):
+    link = service['metadata']['selfLink']
+
+    fields = obj_lbaas.LBaaSPortSpec.fields
+    svc_port_set = {tuple(port[attr] for attr in fields)
+                    for port in get_service_ports(service)}
+
+    spec_port_set = {tuple(getattr(port, attr)
+                     for attr in fields
+                     if port.obj_attr_is_set(attr))
+                     for port in lbaas_spec.ports}
+
+    if svc_port_set != spec_port_set:
+        LOG.debug("LBaaS spec ports %(spec_ports)s != %(svc_ports)s "
+                  "for %(link)s" % {'spec_ports': spec_port_set,
+                                    'svc_ports': svc_port_set,
+                                    'link': link})
+    return svc_port_set != spec_port_set
+
+
+def get_service_ports(service):
+    return [{'name': port.get('name'),
+             'protocol': port.get('protocol', 'TCP'),
+             'port': port['port'],
+             'targetPort': str(port['targetPort'])}
+            for port in service['spec']['ports']]
